/* 
 * Copyright 2015 by AVM GmbH <info@avm.de>
 *
 * This software contains free software; you can redistribute it and/or modify 
 * it under the terms of the GNU General Public License ("License") as 
 * published by the Free Software Foundation  (version 3 of the License). 
 * This software is distributed in the hope that it will be useful, but 
 * WITHOUT ANY WARRANTY; without even the implied warranty of 
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the copy of the 
 * License you received along with this software for more details.
 */

package de.avm.android.tr064.soap.remoteaccess;

import java.io.IOException;
import de.avm.android.tr064.exceptions.BaseException;
import de.avm.android.tr064.exceptions.DataMisformatException;
import de.avm.android.tr064.model.RemoteAccessInfo;
import de.avm.android.tr064.soap.ISoapCredentials;

public class GetInfo extends RemoteAccessSoapHelper<RemoteAccessInfo>
{
	/**
	 * Gets remote access info
	 * 
	 * @param soapCredentials
	 */
	public GetInfo(ISoapCredentials soapCredentials)
	{
		super(soapCredentials);
	}

	@Override
	public RemoteAccessInfo getQualifiedResult()
			throws DataMisformatException, BaseException, IOException
	{
		String input = getSoapBody();

		RemoteAccessInfo result = new RemoteAccessInfo();
		result.setEnabled(getBooleanByName("NewEnabled", input));
		result.setPort(getIntByName("NewPort", input));
		result.setUsername(getValueByName("NewUsername", input));

		return result;
	}

    @Override
    protected String filterSoapBeforeTrace(String body)
    {
        return replaceSecrets(body, new String[] { "NewUsername" });
    }

	@Override
	public String getSoapMethod()
	{
		return "GetInfo";
	}
}
