/* 
 * Copyright 2014 by AVM GmbH <info@avm.de>
 *
 * This software contains free software; you can redistribute it and/or modify 
 * it under the terms of the GNU General Public License ("License") as 
 * published by the Free Software Foundation  (version 3 of the License). 
 * This software is distributed in the hope that it will be useful, but 
 * WITHOUT ANY WARRANTY; without even the implied warranty of 
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the copy of the 
 * License you received along with this software for more details.
 */

package de.avm.android.tr064.soap.wlanconfiguration;

import java.io.IOException;
import de.avm.android.tr064.exceptions.BaseException;
import de.avm.android.tr064.exceptions.DataMisformatException;
import de.avm.android.tr064.soap.ISoapCredentials;

public class SetBeaconType extends WLANSoapHelper<Boolean>
{
	
	private String mBeaconType;
	
	/**
	 * Sets WLAN BeaconType
	 * 	unencrypted  "None"
	 * 	WPA  		"WPA"
	 * 	WPA+WPA2 	"WPAand11i" 
	 * 	WPA2 		"11i"
	 * 
	 * @param interfaceIndex
	 * 		Index of WLAN configuration interface
	 * @param beaconType
	 * 		The new kind of security: 
	 * @throws IllegalArgumentException
	 * 		illegal value of a member of mode 
	 */
	
	public SetBeaconType(ISoapCredentials soapCredentials, int interfaceIndex,
			String beaconType)
	{
		super(soapCredentials, interfaceIndex);
		
		mBeaconType = beaconType;
	}

	@Override
	public Boolean getQualifiedResult()
			throws DataMisformatException, BaseException, IOException
	{
		// no out parameter, check for error response
		exceptOnFault(getSoapBody());
		return Boolean.TRUE;
	}

	@Override
	public String getSoapMethodParameter()
	{
		return "<NewBeaconType>" + mBeaconType + "</NewBeaconType>";
	}

	@Override
	public String getSoapMethod()
	{
		return "SetBeaconType";
	}
}
